@@ -63,6 +63,12 @@ def unicode_quote(string, safe='/'):
     return quote(to_utf8(string), to_utf8(safe))
 
 
+def unicode_quote_path_segment(string):
+    if string is None:
+        return None
+    return quote(to_utf8(string), safe=to_utf8(""))
+
+
 def unicode_unquote(string):
     if string is None:
         return None
@@ -96,14 +102,18 @@ def parse(url_str):
     Extract all parts from a URL string and return them as a dictionary
     """
     url_str = to_unicode(url_str)
-
     result = urlparse(url_str)
     netloc_parts = result.netloc.split('@')
     if len(netloc_parts) == 1:
         username = password = None
         host = netloc_parts[0]
     else:
-        username, password = netloc_parts[0].split(':')
+        user_and_pass = netloc_parts[0].split(':')
+        if len(user_and_pass) == 2:
+            username, password = user_and_pass
+        elif len(user_and_pass) == 1:
+            username = user_and_pass[0]
+            password = None
         host = netloc_parts[1]
 
     if host and ':' in host:
@@ -211,6 +221,8 @@ class URL(object):
         url = self._tuple
         if url.username and url.password:
             netloc = '%s:%s@%s' % (url.username, url.password, url.host)
+        elif url.username and not url.password:
+            netloc = '%s@%s' % (url.username, url.host)
         else:
             netloc = url.host
         if url.port:
@@ -350,7 +362,7 @@ class URL(object):
         """
         if value is not None:
             segments = list(self.path_segments())
-            segments[index] = value
+            segments[index] = unicode_quote_path_segment(value)
             new_path = '/' + '/'.join(segments)
             if self._tuple.path.endswith('/'):
                 new_path += '/'
@@ -367,7 +379,7 @@ class URL(object):
         :param list value: the new path segments to use
         """
         if value is not None:
-            encoded_values = map(unicode_quote, value)
+            encoded_values = map(unicode_quote_path_segment, value)
             new_path = '/' + '/'.join(encoded_values)
             return URL._mutate(self, path=new_path)
         parts = self._tuple.path.split('/')
